@@ -62,8 +62,6 @@
 #define GEDIT_MAX_PATH_LEN  2048
 #endif
 
-#define DEFAULT_ENCODING "ISO-8859-15"
-
 struct _GeditDocumentPrivate
 {
 	gchar		*uri;
@@ -695,7 +693,7 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 {
 	char* file_contents;
 	GnomeVFSResult res;
-   	gsize file_size;
+   	int file_size;
 	GtkTextIter iter, end;
 	
 	gedit_debug (DEBUG_DOCUMENT, "File to load: %s", uri);
@@ -716,84 +714,21 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 
 	if (file_size > 0)
 	{
-		if (!g_utf8_validate (file_contents, file_size, NULL))
+		gchar *converted_text;
+
+		converted_text = gedit_utils_convert_to_utf8 (file_contents,
+							&doc->priv->encoding);
+
+		if (converted_text == NULL)
 		{
-			/* The file contains invalid UTF8 data */
-			/* Try to convert it to UTF-8 from currence locale */
-			GError *conv_error = NULL;
-			gchar* converted_file_contents = NULL;
-			gsize bytes_written;
-			
-			converted_file_contents = g_locale_to_utf8 (file_contents, file_size,
-					NULL, &bytes_written, &conv_error); 
-						
-			if ((conv_error != NULL) || 
-			    !g_utf8_validate (converted_file_contents, bytes_written, NULL))		
-			{
-				/* Coversion failed */	
-				if (conv_error != NULL) {
-					g_error_free (conv_error);
-					conv_error = NULL;
-				}
+			/* bail out */
+			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR,
+				     GEDIT_ERROR_INVALID_UTF8_DATA,
+				     _("Invalid UTF-8 data"));
 
-				if (converted_file_contents != NULL)
-					g_free (converted_file_contents);
-				
-				/* Try to convert it to UTF-8 from default encoding */
-				converted_file_contents = g_convert (file_contents, file_size, 
-						"UTF-8", DEFAULT_ENCODING,
-						NULL, &bytes_written, &conv_error); 
-							
-				if ((conv_error != NULL) || 
-			    		!g_utf8_validate (converted_file_contents, bytes_written, NULL))		
-				{
-					/* Coversion failed */	
-					if (conv_error != NULL)
-						g_error_free (conv_error);
-
-					g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, 
-						     GEDIT_ERROR_INVALID_UTF8_DATA,
-					     	     _("Invalid UTF-8 data"));
-				
-					if (converted_file_contents != NULL)
-						g_free (converted_file_contents);
-				
-					g_free (file_contents);
-
-					return FALSE;
-				}
-				else
-				{
-					if (doc->priv->encoding != NULL)
-						g_free (doc->priv->encoding);	
-				
-					doc->priv->encoding = g_strdup (DEFAULT_ENCODING);
-				}
-					
-			}
-			else
-			{
-				const gchar *encoding = NULL;
-				g_get_charset(&encoding);
-				
-				if (doc->priv->encoding != NULL)
-					g_free (doc->priv->encoding);	
-				
-				doc->priv->encoding = g_strdup (encoding);
-			}
-			
 			g_free (file_contents);
 
-			file_contents = converted_file_contents;
-			file_size = bytes_written;
-		}
-		else
-		{
-			if (doc->priv->encoding != NULL)
-			{
-				g_free (doc->priv->encoding);	
-				doc->priv->encoding = NULL;
-			}
+			return FALSE;
 		}
 
 		gedit_debug (DEBUG_DOCUMENT, "Document encoding: %s", 
@@ -802,7 +737,8 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 		gedit_undo_manager_begin_not_undoable_action (doc->priv->undo_manager);
 		/* Insert text in the buffer */
 		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &iter, 0);
-		gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc), &iter, file_contents, file_size);
+		gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc), &iter, converted_text, file_size);
+		g_free (converted_text);
 
 		/* We had a newline in the buffer to begin with. (The buffer always contains
    		 * a newline, so we delete to the end of the buffer to clean up. */
@@ -840,97 +776,41 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 	return TRUE;
 }
 
-#define GEDIT_STDIN_BUFSIZE 1024
-
 gboolean
 gedit_document_load_from_stdin (GeditDocument* doc, GError **error)
 {
-	GString * file_contents;
-	gchar *tmp_buf = NULL;
-	struct stat stats;
-	guint buffer_length;
-
+	gchar *stdin_data;
 	GtkTextIter iter, end;
-	GnomeVFSResult	res;
 	
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_val_if_fail (doc != NULL, FALSE);
 	
-	fstat (STDIN_FILENO, &stats);
-	
-	if (stats.st_size  == 0)
-		return FALSE;
+	stdin_data = gedit_utils_get_stdin ();
 
-	tmp_buf = g_new0 (gchar, GEDIT_STDIN_BUFSIZE + 1);
-	g_return_val_if_fail (tmp_buf != NULL, FALSE);
-
-	file_contents = g_string_new (NULL);
-	
-	while (feof (stdin) == 0)
+	if (stdin_data != NULL && strlen (stdin_data) > 0)
 	{
-		buffer_length = fread (tmp_buf, 1, GEDIT_STDIN_BUFSIZE, stdin);
-		tmp_buf [buffer_length] = '\0';
-		g_string_append (file_contents, tmp_buf);
+		gchar *converted_text;
 
-		if (ferror (stdin) != 0)
+		converted_text = gedit_utils_convert_to_utf8 (stdin_data,
+							&doc->priv->encoding);
+
+		if (converted_text == NULL)
 		{
-			res = gnome_vfs_result_from_errno (); 
-		
-			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, res,
-				gnome_vfs_result_to_string (res));
+			/* bail out */
+			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR,
+				     GEDIT_ERROR_INVALID_UTF8_DATA,
+				     _("Invalid UTF-8 data"));
+
+			g_free (stdin_data);
 
-			g_free (tmp_buf);
-			g_string_free (file_contents, TRUE);
 			return FALSE;
 		}
-	}
-
-	fclose (stdin);
-
-	if (file_contents->len > 0)
-	{
-		if (!g_utf8_validate (file_contents->str, file_contents->len, NULL))
-		{
-			/* The file contains invalid UTF8 data */
-			/* Try to convert it to UTF-8 from currence locale */
-			GError *conv_error = NULL;
-			gchar* converted_file_contents = NULL;
-			gsize bytes_written;
-			
-			converted_file_contents = g_locale_to_utf8 (file_contents->str, file_contents->len,
-					NULL, &bytes_written, &conv_error); 
-						
-			if ((conv_error != NULL) || 
-			    !g_utf8_validate (converted_file_contents, bytes_written, NULL))		
-			{
-
-				/* Coversion failed */	
-				if (conv_error != NULL)
-					g_error_free (conv_error);
-
-				g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, 
-					     GEDIT_ERROR_INVALID_UTF8_DATA,
-				     	     _("Invalid UTF-8 data"));
-				
-				if (converted_file_contents != NULL)
-					g_free (converted_file_contents);
-				
-				g_string_free (file_contents, TRUE);
-				
-				return FALSE;
-			}
-
-			g_string_free (file_contents, TRUE);
-
-			/* FIXME: this could be more efficient */
-			file_contents = g_string_new (converted_file_contents);
-		}
 
 		gedit_undo_manager_begin_not_undoable_action (doc->priv->undo_manager);
 		/* Insert text in the buffer */
 		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &iter, 0);
-		gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc), &iter, file_contents->str, file_contents->len);
+		gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc), &iter, converted_text, -1);
 
 		/* We had a newline in the buffer to begin with. (The buffer always contains
    		 * a newline, so we delete to the end of the buffer to clean up. */
@@ -942,10 +822,9 @@ gedit_document_load_from_stdin (GeditDocument* doc, GError **error)
 		gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (doc), &iter);
 
 		gedit_undo_manager_end_not_undoable_action (doc->priv->undo_manager);
+		g_free (converted_text);
 	}
 
-	g_string_free (file_contents, TRUE);
-
 	gedit_document_set_readonly (doc, FALSE);
 	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc), TRUE);
 
@@ -1218,9 +1097,7 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	gint fd;
 	gint retval;
 	GeditSaveEncodingSetting encoding_setting;
-	gboolean res;
-	gboolean add_cr;
-
+	
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_val_if_fail (doc != NULL, FALSE);
@@ -1304,9 +1181,9 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 		create_backup_copy = FALSE;
 
 		/* Use default permissions */
-		saved_umask = umask (0);
+		saved_umask = umask(0);
 		st.st_mode = 0666 & ~saved_umask;
-		umask (saved_umask);
+		umask(saved_umask);
 		st.st_uid = getuid ();
 
 		result = stat (dirname, &dir_st);
@@ -1334,7 +1211,7 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 		goto out;
 	}
 
-      	chars = gedit_document_get_buffer (doc);
+	chars = gedit_document_get_chars (doc, 0, -1);
 
 	encoding_setting = gedit_prefs_manager_get_save_encoding ();
 
@@ -1391,17 +1268,7 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	}	    
 
 	chars_len = strlen (chars);
-
-	add_cr = (*(chars + chars_len - 1) != '\n');
-
-	/* Save the file content */
-	res = (write (fd, chars, chars_len) == chars_len);
-
-	if (res && add_cr)
-		/* Add \n if needed */
-		res = (write (fd, "\n", 1) == 1);
-
-	if (!res)
+	if (write (fd, chars, chars_len) != chars_len)
 	{
 		gchar *msg;
 
@@ -1499,21 +1366,6 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	return retval;
 }
 
-gchar* 
-gedit_document_get_buffer (const GeditDocument *doc)
-{
-	GtkTextIter start, end;
-
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_val_if_fail (doc != NULL, FALSE);
-
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &start, 0);
-      	gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end);
-  
-      	return gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc), &start, &end, TRUE);
-}
-
 gboolean	
 gedit_document_is_untitled (const GeditDocument* doc)
 {
@@ -1573,21 +1425,6 @@ gedit_document_get_line_count (const GeditDocument *doc)
 	return gtk_text_buffer_get_line_count (GTK_TEXT_BUFFER (doc));
 }
 
-void 
-gedit_document_delete_all_text (GeditDocument *doc)
-{
-	GtkTextIter start, end;
-
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_if_fail (doc != NULL);
-
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &start, 0);
-      	gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end);
-	
-	gtk_text_buffer_delete (GTK_TEXT_BUFFER (doc), &start, &end);
-}
-
 gboolean 
 gedit_document_revert (GeditDocument *doc,  GError **error)
 {
@@ -1604,11 +1441,11 @@ gedit_document_revert (GeditDocument *doc,  GError **error)
 		return FALSE;
 	}
 			
-	buffer = gedit_document_get_buffer (doc);
+	buffer = gedit_document_get_chars (doc, 0, -1);
 
 	gedit_undo_manager_begin_not_undoable_action (doc->priv->undo_manager);
 
-	gedit_document_delete_all_text (doc);
+	gedit_document_delete_text (doc, 0, -1);
 
 	if (!gedit_document_load (doc, doc->priv->uri, error))
 	{
@@ -1672,10 +1509,14 @@ gedit_document_delete_text (GeditDocument *doc, gint start, gint end)
 
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 	g_return_if_fail (start >= 0);
-	g_return_if_fail (end >= 0);
+	g_return_if_fail ((end > start) || end < 0);
 
 	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &start_iter, start);
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &end_iter, end);
+
+	if (end < 0)
+		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end_iter);
+	else
+		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &end_iter, end);
 
 	gtk_text_buffer_delete (GTK_TEXT_BUFFER (doc), &start_iter, &end_iter);
 }
@@ -1690,10 +1531,14 @@ gedit_document_get_chars (GeditDocument *doc, gint start, gint end)
 
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
 	g_return_val_if_fail (start >= 0, NULL);
-	g_return_val_if_fail (end >= 0, NULL);
+	g_return_val_if_fail ((end > start) || (end < 0), NULL);
 
 	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &start_iter, start);
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &end_iter, end);
+
+	if (end < 0)
+		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end_iter);
+	else
+		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &end_iter, end);
 
 	return gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc), &start_iter, &end_iter, TRUE);
 }
@@ -1856,6 +1701,36 @@ gedit_document_get_last_replace_text (GeditDocument* doc)
 		g_strdup (doc->priv->last_replace_text) : NULL;	
 }
 
+void 
+gedit_document_set_last_searched_text (GeditDocument* doc, const gchar *text)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+	g_return_if_fail (doc->priv != NULL);
+	g_return_if_fail (text != NULL);
+
+	if (doc->priv->last_searched_text != NULL)
+		g_free (doc->priv->last_searched_text);
+
+	doc->priv->last_searched_text = g_strdup (text);
+}
+
+void 
+gedit_document_set_last_replace_text (GeditDocument* doc, const gchar *text)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+	g_return_if_fail (doc->priv != NULL);
+	g_return_if_fail (text != NULL);
+
+	if (doc->priv->last_replace_text != NULL)
+		g_free (doc->priv->last_replace_text);
+
+	doc->priv->last_replace_text = g_strdup (text);
+}
+
 gboolean
 gedit_document_find (GeditDocument* doc, const gchar* str, 
 		gboolean from_cursor, gboolean case_sensitive)
@@ -1954,44 +1829,8 @@ gedit_document_find_again (GeditDocument* doc)
 	return found;
 }
 
-gchar*
-gedit_document_get_selected_text (GeditDocument *doc, gint *start, gint *end)
-{
-	GtkTextIter iter;
-	GtkTextIter sel_bound;
-
-	gedit_debug (DEBUG_DOCUMENT, "");
-
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
-
-	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
-                                    &iter,
-                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
-					                      "insert"));
-		
-	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
-                                    &sel_bound,
-                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
-					                      "selection_bound"));
-	gtk_text_iter_order (&iter, &sel_bound);	
-
-	if (start != NULL)
-		*start = gtk_text_iter_get_offset (&iter); 
-
-	if (end != NULL)
-		*end = gtk_text_iter_get_offset (&sel_bound); 
-
-	if (gtk_text_iter_equal (&sel_bound, &iter))
-	{
-		gedit_debug (DEBUG_DOCUMENT, "There is no selected text");
-
-		return NULL;
-	}
-	return gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc), &iter, &sel_bound, TRUE);
-}
-
-gboolean
-gedit_document_has_selected_text (GeditDocument *doc)
+gboolean 
+gedit_document_get_selection (GeditDocument *doc, gint *start, gint *end)
 {
 	GtkTextIter iter;
 	GtkTextIter sel_bound;
@@ -2009,6 +1848,13 @@ gedit_document_has_selected_text (GeditDocument *doc)
                                     &sel_bound,
                                     gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
 					                      "selection_bound"));
+	gtk_text_iter_order (&iter, &sel_bound);	
+
+	if (start != NULL)
+		*start = gtk_text_iter_get_offset (&iter); 
+
+	if (end != NULL)
+		*end = gtk_text_iter_get_offset (&sel_bound); 
 
 	return !gtk_text_iter_equal (&sel_bound, &iter);	
 }
@@ -2141,3 +1987,28 @@ gedit_document_set_cursor (GeditDocument *doc, gint cursor)
 	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &iter, cursor);
 	gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (doc), &iter);
 }
+
+void
+gedit_document_set_selection (GeditDocument *doc, gint start, gint end)
+{
+ 	GtkTextIter start_iter;
+	GtkTextIter end_iter;
+
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+	g_return_if_fail (start >= 0);
+	g_return_if_fail ((end > start) || (end < 0));
+
+	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &start_iter, start);
+
+	if (end < 0)
+		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end_iter);
+	else
+		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &end_iter, end);
+
+	gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (doc), &end_iter);
+
+	gtk_text_buffer_move_mark_by_name (GTK_TEXT_BUFFER (doc),
+					"selection_bound", &start_iter);
+}
