@@ -35,6 +35,9 @@
 #include <eel/eel-vfs-extensions.h>
 #include <eel/eel-string.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 
 #include <string.h>
 
@@ -42,6 +45,9 @@
 #include "gedit2.h"
 #include "bonobo-mdi.h"
 #include "gedit-document.h"
+#include "gedit-debug.h"
+
+#define DEFAULT_ENCODING "ISO-8859-15"
 
 /* =================================================== */
 /* Flash */
@@ -212,15 +218,28 @@ gedit_utils_is_uri_read_only (const gchar* uri)
 
 	gint res;
 
+	g_return_val_if_fail (uri != NULL, TRUE);
+	
+	gedit_debug (DEBUG_FILE, "URI: %s", uri);
+
 	/* FIXME: all remote files are marked as readonly */
 	if (!gedit_utils_uri_has_file_scheme (uri))
 		return TRUE;
 			
 	canonical_uri = eel_make_uri_canonical (uri);
-	file_uri = gnome_vfs_unescape_string_for_display (canonical_uri);
+	g_return_val_if_fail (canonical_uri != NULL, TRUE);
 	
-	/* Note that: strlen ("file://") == 7 */	
-	res = access (file_uri + 7, W_OK);
+	gedit_debug (DEBUG_FILE, "CANONICAL URI: %s", canonical_uri);
+
+	file_uri = gnome_vfs_get_local_path_from_uri (canonical_uri);
+	if (file_uri == NULL)
+	{
+		gedit_debug (DEBUG_FILE, "FILE URI: NULL");
+
+		return TRUE;
+	}
+	
+	res = access (file_uri, W_OK);
 
 	g_free (canonical_uri);
 	g_free (file_uri);
@@ -377,7 +396,7 @@ finally_1:
  * n: len of the string in bytes
  */
 gboolean 
-g_uft8_caselessnmatch (const char *s1, const char *s2, gssize n1, gssize n2)
+g_utf8_caselessnmatch (const char *s1, const char *s2, gssize n1, gssize n2)
 {
 	gchar *casefold;
 	gchar *normalized_s1;
@@ -511,7 +530,7 @@ lines_match (const GtkTextIter *start,
       /* If it's not the first line, we have to match from the
        * start of the line.
        */
-      if (g_uft8_caselessnmatch (line_text, *lines, strlen (line_text), strlen (*lines)) == 0)
+      if (g_utf8_caselessnmatch (line_text, *lines, strlen (line_text), strlen (*lines)) == 0)
         found = line_text;
       else
         found = NULL;
@@ -1246,6 +1265,100 @@ gedit_utils_error_reporting_reverting_file (
 	g_free (error_message);
 }
 
+void
+gedit_utils_error_reporting_creating_file (
+		const gchar *uri,
+		gint error_code,
+		GtkWindow *parent)
+{
+	gchar *error_message;
+	gchar *full_formatted_uri;
+       	gchar *uri_for_display	;
+	
+	GtkWidget *dialog;
+
+	g_return_if_fail (uri != NULL);
+	
+	full_formatted_uri = eel_format_uri_for_display (uri);
+
+	/* Truncate the URI so it doesn't get insanely wide. Note that even
+	 * though the dialog uses wrapped text, if the URI doesn't contain
+	 * white space then the text-wrapping code is too stupid to wrap it.
+	 */
+        uri_for_display = eel_str_middle_truncate (full_formatted_uri, 
+			MAX_URI_IN_DIALOG_LENGTH);
+	g_free (full_formatted_uri);
+
+	switch (error_code)
+	{
+		case EEXIST: 
+			error_message = g_strdup_printf (_("The file \"%s\" already exists."),
+				uri_for_display);
+			break;
+
+		case EISDIR:
+			error_message = g_strdup_printf (
+				_("\"%s\" is a directory.\n\n"
+				  "Please, check that you typed the location correctly."),
+                         	uri_for_display);
+                	break;
+
+		case EACCES:
+		case EROFS:
+		case ETXTBSY:
+			error_message = g_strdup_printf (
+				_("Cannot create the file \"%s\".\n\n"
+				  "Make sure you have the appropriate write permissions."),
+				uri_for_display);
+			break;
+
+		case ENAMETOOLONG:
+			error_message = g_strdup_printf (
+				_("Cannot create the file \"%s\".\n\n"
+				  "The file name is too long."),
+				uri_for_display);
+			break;
+		
+		case ENOENT:
+			error_message = g_strdup_printf (
+				_("Cannot create the file \"%s\".\n\n"
+				  "A directory component in the file name does not exist or "
+		                  "is a dangling symbolic link."),
+				uri_for_display);	
+			break;
+
+
+		case ENOSPC:
+			error_message = g_strdup_printf (
+				 _("There is not enough disk space to create the file \"%s\".\n\n"
+				   "Please free some disk space and try again."),
+				 uri_for_display);
+			break;
+
+		default:
+			error_message = g_strdup_printf (_("Could not create the file \"%s\"."),
+							 uri_for_display);
+	}
+	
+	dialog = gtk_message_dialog_new (
+			parent,
+			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+		   	GTK_MESSAGE_ERROR,
+		   	GTK_BUTTONS_OK,
+			error_message);
+			
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+
+	g_free (uri_for_display);
+	g_free (error_message);
+}
+
+
 /*
  * gedit_utils_set_atk_name_description
  * @widget : The Gtk widget for which name/description to be set
@@ -1310,9 +1423,11 @@ gedit_utils_uri_exists (const gchar* text_uri)
 {
 	GnomeVFSURI *uri;
 	gboolean res;
-	
+		
 	g_return_val_if_fail (text_uri != NULL, FALSE);
 	
+	gedit_debug (DEBUG_FILE, "text_uri: %s", text_uri);
+
 	uri = gnome_vfs_uri_new (text_uri);
 	g_return_val_if_fail (uri != NULL, FALSE);
 
@@ -1320,5 +1435,249 @@ gedit_utils_uri_exists (const gchar* text_uri)
 
 	gnome_vfs_uri_unref (uri);
 
+	gedit_debug (DEBUG_FILE, res ? "TRUE" : "FALSE");
+
 	return res;
 }
+
+gchar *
+gedit_utils_convert_search_text (const gchar *text)
+{
+	GString *str;
+	gint length;
+	gboolean drop_prev = FALSE;
+	const gchar *cur;
+	const gchar *end;
+	const gchar *prev;
+
+	g_return_val_if_fail (text != NULL, NULL);
+
+	length = strlen (text);
+
+	str = g_string_new ("");
+
+	cur = text;
+	end = text + length;
+	prev = NULL;
+	while (cur != end) {
+		const gchar *next;
+		next = g_utf8_next_char (cur);
+
+		if (prev && (*prev == '\\')) {
+			switch (*cur) {
+				case 'n':
+					str = g_string_append (str, "\n");
+				break;
+				case 'r':
+					str = g_string_append (str, "\r");
+				break;
+				case 't':
+					str = g_string_append (str, "\t");
+				break;
+				case '\\':
+					str = g_string_append (str, "\\");
+					drop_prev = TRUE;
+				break;
+				default:
+					str = g_string_append (str, "\\");
+					str = g_string_append (str, cur);
+				break;
+			}
+		} else if (*cur != '\\') {
+			str = g_string_append_len (str, cur, next - cur);
+		}
+
+		if (!drop_prev)
+			prev = cur;
+		else {
+			prev = NULL;
+			drop_prev = FALSE;
+		}
+
+		cur = next;
+	}
+
+	return g_string_free (str, FALSE);
+}
+
+gboolean
+gedit_utils_create_empty_file (const gchar *uri)
+{
+	gchar *canonical_uri;
+	gchar *filename;
+	int fd;
+	
+	g_return_val_if_fail (uri != NULL, FALSE);
+	
+	/* Get filename from uri */
+	if (!gedit_utils_uri_has_file_scheme (uri))
+		return FALSE;
+			
+	canonical_uri = eel_make_uri_canonical (uri);
+	g_return_val_if_fail (canonical_uri != NULL, FALSE);
+	
+	gedit_debug (DEBUG_FILE, "CANONICAL URI: %s", canonical_uri);
+
+	filename = gnome_vfs_get_local_path_from_uri (canonical_uri);
+	g_free (canonical_uri);
+	
+	if (filename == NULL)
+	{
+		gedit_debug (DEBUG_FILE, "FILENAME: NULL");
+
+		return FALSE;
+	}
+	
+	fd = open (filename, O_CREAT | O_EXCL | O_WRONLY, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+	g_free (filename);
+	
+	if (fd == -1)
+		return FALSE;
+	
+	return (close (fd) == 0);
+}
+
+
+
+
+#define GEDIT_STDIN_BUFSIZE 1024
+
+gchar *
+gedit_utils_get_stdin (void)
+{
+	GString * file_contents;
+	gchar *tmp_buf = NULL;
+	struct stat stats;
+	guint buffer_length;
+	GnomeVFSResult	res;
+	
+	fstat (STDIN_FILENO, &stats);
+	
+	if (stats.st_size  == 0)
+		return NULL;
+
+	tmp_buf = g_new0 (gchar, GEDIT_STDIN_BUFSIZE + 1);
+	g_return_val_if_fail (tmp_buf != NULL, FALSE);
+
+	file_contents = g_string_new (NULL);
+	
+	while (feof (stdin) == 0)
+	{
+		buffer_length = fread (tmp_buf, 1, GEDIT_STDIN_BUFSIZE, stdin);
+		tmp_buf [buffer_length] = '\0';
+		g_string_append (file_contents, tmp_buf);
+
+		if (ferror (stdin) != 0)
+		{
+			res = gnome_vfs_result_from_errno (); 
+		
+			g_free (tmp_buf);
+			g_string_free (file_contents, TRUE);
+			return NULL;
+		}
+	}
+
+	fclose (stdin);
+
+	return g_string_free (file_contents, FALSE);
+}
+
+
+
+gchar *
+gedit_utils_convert_to_utf8 (const gchar *content, gchar **encoding)
+{
+	gchar *utf8_content = NULL;
+
+	g_return_val_if_fail (content != NULL, NULL);
+
+	if (strlen (content) > 0)
+	{
+		gedit_debug (DEBUG_UTILS, "Checking to see if input is valid UTF-8");
+
+		if (!g_utf8_validate (content, -1, NULL))
+		{
+			/* The file contains invalid UTF8 data */
+			/* Try to convert it to UTF-8 from currence locale */
+			GError *conv_error = NULL;
+			gchar* converted_contents = NULL;
+			gsize bytes_written;
+
+			gedit_debug (DEBUG_UTILS, "Trying to convert from locale to UTF-8");
+			
+			converted_contents = g_locale_to_utf8 (content, -1,
+					NULL, &bytes_written, &conv_error); 
+						
+			if ((conv_error != NULL) || 
+			    !g_utf8_validate (converted_contents, bytes_written, NULL))		
+			{
+				gedit_debug (DEBUG_UTILS,
+				    "Couldn't convert from locale to UTF-8.");
+				
+				if (converted_contents != NULL)
+					g_free (converted_contents);
+
+				if (conv_error != NULL)
+				{
+					g_error_free (conv_error);
+					conv_error = NULL;
+				}
+				
+				/* try to convert from the default encoding */
+				converted_contents =
+					g_convert (content, strlen (content), 
+						"UTF-8", DEFAULT_ENCODING,
+						NULL, &bytes_written,
+						&conv_error); 
+
+				if ((conv_error != NULL) ||
+				    !g_utf8_validate (converted_contents,
+					    	      bytes_written, NULL))
+				{
+					gedit_debug (DEBUG_UTILS,
+					    "Couldn't convert from %s to UTF-8.", DEFAULT_ENCODING);
+					/* Coversion failed */	
+					if (conv_error != NULL)
+						g_error_free (conv_error);
+
+					if (converted_contents != NULL)
+						g_free (converted_contents);
+				
+					return NULL;
+				}
+				else
+				{
+					gedit_debug (DEBUG_UTILS,
+						"Converted from %s to UTF-8.", DEFAULT_ENCODING);
+					if (encoding != NULL)
+						*encoding = g_strdup (DEFAULT_ENCODING);
+				}
+			}
+			else
+			{
+				/* successfully converted from their locale
+				 * to utf8
+				 */
+				const gchar *charset;
+
+				g_get_charset (&charset);
+
+				if (encoding != NULL)
+					*encoding = g_strdup (charset);
+
+				gedit_debug (DEBUG_UTILS,
+					"Converted from locale to UTF-8.");
+			}
+
+			utf8_content = converted_contents;
+		}
+		else
+		{
+			/* they gave us valid UTF-8 */
+			utf8_content = g_strdup (content);
+		}
+	}
+
+	return utf8_content;
+}
